using System;
using System.Windows.Forms;
using Demo_WinDLL.Properties;
using EposPCmd.Net;

namespace Demo_WinDLL
{
    public partial class Form1 : Form
    {
        #region private members

        bool _updateActive;
        DeviceManager _deviceManager;
        Plc _plc;

        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            
            _updateActive = false;
        }

        /// <summary>
        /// Handles the Click event of the buttonExit control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonExitClick(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;

            Close();
        }

        /// <summary>
        /// Handles the Load event of the Form1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Form1_Load(object sender, EventArgs e)
        {
            InitSettings(true);
        }

        /// <summary>
        /// Handles the Click event of the buttonSettings control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonSettingsClick(object sender, EventArgs e)
        {
            InitSettings(true);
        }

        /// <summary>
        /// Starts the timer.
        /// </summary>
        /// <returns></returns>
        private void StartTimer()
        {
            if (!timerStatus.Enabled)
            {
                timerStatus.Interval = 1000;
                timerStatus.Start();

                _updateActive = true;
            }

            return;
        }

        /// <summary>
        /// Stops the timer.
        /// </summary>
        /// <returns></returns>
        private bool StopTimer()
        {
            timerStatus.Stop();

            _updateActive = false;

            return true;
        }


        /// <summary>
        /// Closes the settings.
        /// </summary>
        /// <returns></returns>
        private bool CloseSettings()
        {
            if (!StopTimer())
                return false;

            if (!ClosePlcSettings())
                return false;

            return CloseCommSettings();
        }

        /// <summary>
        /// Closes the PLC settings.
        /// </summary>
        /// <returns></returns>
        private bool ClosePlcSettings()
        {
            bool retVal = false;

	        try
            {
                if( _plc!=null )
                    _plc.Dispose();
                _plc = null;

                retVal = true;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message,
                                 Application.ProductName,
                                 MessageBoxButtons.OK,
                                 MessageBoxIcon.Error);
            }


            return retVal;
        }

        /// <summary>
        /// Closes the comm settings.
        /// </summary>
        /// <returns></returns>
        private bool CloseCommSettings()
        {
            bool retVal = false;

            try
            {
                if( _deviceManager!=null )
                    _deviceManager.Dispose();

                _deviceManager = null;
                
                retVal = true;
            }
            catch (Exception e)
            {
                MessageBox.Show( e.Message, 
                                 Application.ProductName, 
                                 MessageBoxButtons.OK, 
                                 MessageBoxIcon.Error );
            }

            return retVal;
        }

        /// <summary>
        /// Opens the comm settings.
        /// </summary>
        /// <param name="showMsg">if set to <c>true</c> [show MSG].</param>
        /// <returns></returns>
        private bool OpenCommSettings(bool showMsg)
        {
            bool retVal = false;

            try
            {
                _deviceManager = new DeviceManager();

                retVal = true;
            }
            catch (Exception e)
            {
                if (showMsg) MessageBox.Show( e.Message,
                                              Application.ProductName,
                                              MessageBoxButtons.OK,
                                              MessageBoxIcon.Error );
            }

            return retVal;
        }

        /// <summary>
        /// Opens the PLC settings.
        /// </summary>
        /// <param name="showMsg">if set to <c>true</c> [show MSG].</param>
        /// <returns></returns>
        private bool OpenPlcSettings(bool showMsg)
        {
            bool retVal = false;
            const string deviceName = "EPOS2 P";

            try
            {
                _plc = _deviceManager.CreatePlc(deviceName);

                _plc.NodeId = Byte.Parse(textBoxNodeId.Text);

                retVal = true;
            }
            catch (Exception e)
            {
                if (showMsg) MessageBox.Show(e.Message,
                                              Application.ProductName,
                                              MessageBoxButtons.OK,
                                              MessageBoxIcon.Error);
            }

	        return retVal;
        }

        /// <summary>
        /// Inits the settings.
        /// </summary>
        /// <param name="showMsg">if set to <c>true</c> [show MSG].</param>
        /// <returns></returns>
        private void InitSettings(bool showMsg)
        {
            if (!CloseSettings())
                return;

            if (!OpenCommSettings(showMsg))
                return;

            if (!OpenPlcSettings(showMsg))
                return;

            StartTimer();
        }

        /// <summary>
        /// Writes the amplitude.
        /// </summary>
        /// <param name="amplitude">The amplitude.</param>
        /// <returns></returns>
        private bool WriteAmplitude(uint amplitude)
        {
            bool result = false;
            
            try
            {
                if (_plc != null)
                {
                    const EProcessInputType processInputType = EProcessInputType.PitProcessInputInt32;
                    const byte elementNumber = 1;
                    byte[]              data = BitConverter.GetBytes(amplitude);
                    uint                nbOfBytesWritten = 0;

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInput(processInputType,
                                                            elementNumber, data, ref nbOfBytesWritten);
                    result = true;
                }
            }
            catch (Exception e)
            {
                result = false;
                textBoxStatus.Text = e.Message;
            }

            return result;
        }


        /// <summary>
        /// Updates the amplitude.
        /// </summary>
        /// <returns></returns>
        private bool UpdateAmplitude()
        {
            return WriteAmplitude(UInt32.Parse(textBoxAmplitude.Text));
        }

        /// <summary>
        /// Starts the cycles.
        /// </summary>
        /// <returns></returns>
        private void StartCycles()
        {
            try
            {
                if (_plc != null)
                {
                    const EProcessInputType processInputType = EProcessInputType.PitProcessInputInt8;
                    const byte elementNumber = 1;
                    const byte bitNumber = 0;
                    const byte bitStateLow = 0;
                    const byte bitStateHigh = 1;
                    
                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType,
                                                            elementNumber, bitNumber, bitStateLow );

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType,
                                                            elementNumber, bitNumber, bitStateHigh);
                    return;
                }
            }
            catch (Exception e)
            {
                textBoxStatus.Text = e.Message;
            }

            return;
        }

        /// <summary>
        /// Stops the cycles.
        /// </summary>
        /// <returns></returns>
        private void StopCycles()
        {
            try
            {
                if (_plc != null)
                {
                    const EProcessInputType processInputType = EProcessInputType.PitProcessInputInt8;
                    const byte elementNumber = 1;
                    const byte bitNumber = 0;
                    const byte bitStateLow = 0;
                    const byte bitStateHigh = 1;

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType,
                                                            elementNumber, bitNumber, bitStateHigh);

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType,
                                                            elementNumber, bitNumber, bitStateLow);
                }
            }
            catch (Exception e)
            {
                textBoxStatus.Text = e.Message;
            }

            return;
        }

        /// <summary>
        /// Resets the error.
        /// </summary>
        /// <returns></returns>
        private void ResetError()
        {
            try
            {
                if (_plc != null)
                {
                    const EProcessInputType processInputType = EProcessInputType.PitProcessInputInt8;
                    const byte elementNumber = 1;
                    const byte bitNumber = 1;
                    const byte bitStateLow = 0;
                    const byte bitStateHigh = 1;

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType,
                                                            elementNumber, bitNumber, bitStateLow);

                    _plc.ProcessControl.ProcessInputOutput.SetProcessInputBit(processInputType,
                                                            elementNumber, bitNumber, bitStateHigh);
                }
            }
            catch (Exception e)
            {
                textBoxStatus.Text = e.Message;
            }

            return;
        }

        /// <summary>
        /// Handles the Click event of the buttonStartCycles control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonStartCyclesClick(object sender, EventArgs e)
        {
            if (UpdateAmplitude())
            {
                StartCycles();
            }
        }

        /// <summary>
        /// Handles the Click event of the buttonStopCycles control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonStopCyclesClick(object sender, EventArgs e)
        {
            StopCycles();
        }

        /// <summary>
        /// Handles the Click event of the buttonResetError control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonResetErrorClick(object sender, EventArgs e)
        {
            ResetError();
        }

        /// <summary>
        /// Handles the Click event of the buttonDownloadProgram control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonDownloadProgramClick(object sender, EventArgs e)
        {
            try
            {
                OpenFileDialog dlg = new OpenFileDialog();

                dlg.Filter = Resources.Filter;

                if (dlg.ShowDialog() == DialogResult.OK)
                {
                    string fileName = dlg.FileName;

                    _plc.ProcessControl.ProgramControl.DownloadProgram(fileName);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,
                                Application.ProductName,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Handles the Click event of the buttonStartProgram control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonStartProgramClick(object sender, EventArgs e)
        {
            try
            {
                _plc.ProcessControl.ProgramControl.ColdstartPlc();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,
                                Application.ProductName,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Handles the Click event of the buttonStopProgram control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonStopProgramClick(object sender, EventArgs e)
        {
            try
            {
                _plc.ProcessControl.ProgramControl.StopPlc();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,
                                Application.ProductName,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Determines whether this instance is error.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance is error; otherwise, <c>false</c>.
        /// </returns>
        private bool IsError()
        {
	        bool result = false;

            try
            {
                if (_plc != null)
                {
                    const EProcessOutputType processOutputType = EProcessOutputType.PitProcessOutputInt8;
                    const byte elementNumber = 1;
                    const byte bitNumber = 1;
                    byte bitState = 0;

                    _plc.ProcessControl.ProcessInputOutput.GetProcessOutputBit(processOutputType,
                                                                                elementNumber,
                                                                                bitNumber, ref bitState);

                    result = bitState > 0;
                }
            }
            catch (Exception e)
            {
                result = true;
                textBoxStatus.Text = e.Message;
            }
        	
	        return result;
        }

        /// <summary>
        /// Determines whether this instance is running.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance is running; otherwise, <c>false</c>.
        /// </returns>
        private bool IsRunning()
        {
            bool result = false;

            try
            {
                if (_plc != null)
                {
                    const EProcessOutputType processOutputType = EProcessOutputType.PitProcessOutputInt8;
                    const byte elementNumber = 1;
                    const byte bitNumber = 0;
                    byte bitState = 0;

                    _plc.ProcessControl.ProcessInputOutput.GetProcessOutputBit(processOutputType,
                                                                                elementNumber,
                                                                                bitNumber, ref bitState);

                    result = bitState > 0;
                }
            }
            catch (Exception e)
            {
                result = false;
                textBoxStatus.Text = e.Message;
            }

            return result;
        }

        /// <summary>
        /// Reads the count.
        /// </summary>
        /// <param name="count">The count.</param>
        /// <returns></returns>
        private bool ReadCount(ref uint count)
        {
	        bool result = true;
            
            count = 0;

            try
            {
                if (_plc != null)
                {
                    const EProcessOutputType processOutputType = EProcessOutputType.PitProcessOutputUint32;
                    const byte elementNumber = 1;
                    byte[]              data = new byte[sizeof(uint)];
                    uint                nbOfBytesRead = 0;
                    
                    _plc.ProcessControl.ProcessInputOutput.GetProcessOutput(processOutputType, 
                                                            elementNumber, ref data, ref nbOfBytesRead);

                    count = BitConverter.ToUInt32(data, 0);
                }
            }
            catch (Exception e)
            {
                result = false;
                textBoxStatus.Text = e.Message;
            }

            return result;
        }

        /// <summary>
        /// Updates the status.
        /// </summary>
        /// <returns></returns>
        private bool UpdateStatus()
        {
            bool retVal = true;

            try
            {
                SuspendLayout();

                if (_plc != null && _updateActive)
                {
                    int isRunning = 0;
                    int isProgramAvailable = 0;

                    _plc.ProcessControl.ProgramControl.GetPlcStatus(ref isRunning, ref isProgramAvailable);

                    textBoxStatus.Text = isProgramAvailable != 0 ? "Program valid" : "No program";

                    buttonDownloadProgram.Enabled = (isProgramAvailable == 0 || isRunning == 0);

                    if (isProgramAvailable != 0)
                    {
                        textBoxStatus.Text = isRunning != 0 ? "Running" : "Not running";

                        if (isRunning != 0)
                        {
                            buttonSettings.Enabled = false;
                            textBoxNodeId.Enabled = false;
                            buttonStopProgram.Enabled = true;
                            buttonStartProgram.Enabled = false;

                            if (IsError())
                            {
                                textBoxStatus.Text = Resources.CycleError;
                                buttonResetError.Enabled = true;
                                buttonStopCycles.Enabled = false;
                                buttonStartCycles.Enabled = false;
                                                            }
                            else if (IsRunning())
                            {
                                textBoxStatus.Text = Resources.CycleRunning;
                                buttonStopProgram.Enabled = true;
                                buttonStopCycles.Enabled = true;
                                buttonStartCycles.Enabled = false;
                            }
                            else
                            {
                                textBoxStatus.Text = Resources.CycleStopped;
                                buttonStartProgram.Enabled = false;
                                buttonStopProgram.Enabled = true;
                                buttonStartCycles.Enabled = true;
                                buttonStopCycles.Enabled = false;
                            }
                        }
                        else
                        {
                            buttonStartProgram.Enabled = true;
                            buttonStopProgram.Enabled = false;
                            buttonStartCycles.Enabled = false;
                            buttonStopCycles.Enabled = false;
                            buttonResetError.Enabled = false;
                            buttonSettings.Enabled = true;
                            textBoxNodeId.Enabled = true;
                            textBoxAmplitude.Enabled = false;
                        }
                    }

                    uint count = 0;

                    textBoxCount.Text = !ReadCount(ref count) ? "0" : String.Format("{0}", count);
                }
                else
                    NoCommunication();                
            }
            catch (Exception)
            {
                retVal = false;
                
                NoCommunication();
            }
            finally
            {
                ResumeLayout();
            }

            return retVal;
        }

        /// <summary>
        /// Noes the communication.
        /// </summary>
        private void NoCommunication()
        {
            textBoxStatus.Text = Resources.NoCommunication;

            buttonStartProgram.Enabled = false;
            buttonStopProgram.Enabled = false;
            buttonStartCycles.Enabled = false;
            buttonStopCycles.Enabled = false;
            buttonResetError.Enabled = false;
            buttonSettings.Enabled = true;
            textBoxNodeId.Enabled = true;
            textBoxAmplitude.Enabled = false;
        }

        /// <summary>
        /// Handles the Tick event of the timerStatus control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void TimerStatusTick(object sender, EventArgs e)
        {
            if (_updateActive && !UpdateStatus())
                StopTimer();
        }

        /// <summary>
        /// Handles the TextChanged event of the textBoxAmplitude control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void TextBoxAmplitudeTextChanged(object sender, EventArgs e)
        {
            UpdateAmplitude();
        }

        /// <summary>
        /// Handles the TextChanged event of the textBoxNodeId control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void TextBoxNodeIdTextChanged(object sender, EventArgs e)
        {
            try
            {
                if (_plc != null)
                {
                    _plc.NodeId = Convert.ToByte(textBoxNodeId.Text);

                    StartTimer();
                }
            }
            catch (Exception ex)
            {
                textBoxStatus.Text = ex.Message;
            }
        }

        /// <summary>
        /// Handles the FormClosing event of the Form1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            CloseSettings();
        }        
    }
}